
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <string.h>

#include <audioresample/resample.h>


static int test_chirp (ResampleFormat format, int i_rate, int o_rate,
    double start_freq, double stop_freq, double expected_error);

int
main (int argc, char *argv[])
{
  int ret;

  resample_init ();

  ret = 1;
  /* s16 */
  ret &= test_chirp (RESAMPLE_FORMAT_S16, 48000, 44100, 0.0, 44100.0/2,
      9.17e-4);
  ret &= test_chirp (RESAMPLE_FORMAT_S16, 44100, 48000, 0.0, 44100.0/2,
      0.0277);

  ret &= test_chirp (RESAMPLE_FORMAT_S16, 48000, 24000, 0.0, 24000.0/2,
      0);
  ret &= test_chirp (RESAMPLE_FORMAT_S16, 48000, 12000, 0.0, 12000.0/2,
      0);
  ret &= test_chirp (RESAMPLE_FORMAT_S16, 48000, 8000, 0.0, 8000.0/2,
      0);
  ret &= test_chirp (RESAMPLE_FORMAT_S16, 48000, 6000, 0.0, 6000.0/2,
      0);

  ret &= test_chirp (RESAMPLE_FORMAT_S16, 6000, 48000, 0.0, 6000.0/2,
      0.0307);
  ret &= test_chirp (RESAMPLE_FORMAT_S16, 12000, 48000, 0.0, 12000.0/2,
      0.0294);
  ret &= test_chirp (RESAMPLE_FORMAT_S16, 24000, 48000, 0.0, 24000.0/2,
      0.0284);

  ret &= test_chirp (RESAMPLE_FORMAT_S16, 8000, 44100, 0.0, 8000.0/2,
      0.030);

  /* s32 */
  ret &= test_chirp (RESAMPLE_FORMAT_S32, 48000, 44100, 0.0, 44100.0/2,
      9.17e-4);
  ret &= test_chirp (RESAMPLE_FORMAT_S32, 44100, 48000, 0.0, 44100.0/2,
      0.0277);
  
  /* f32 */
  ret &= test_chirp (RESAMPLE_FORMAT_F32, 48000, 44100, 0.0, 44100.0/2,
      9.17e-4);
  ret &= test_chirp (RESAMPLE_FORMAT_F32, 44100, 48000, 0.0, 44100.0/2,
      0.0277);
  
  /* f64 */
  ret &= test_chirp (RESAMPLE_FORMAT_F64, 48000, 44100, 0.0, 44100.0/2,
      9.17e-4);
  ret &= test_chirp (RESAMPLE_FORMAT_F64, 44100, 48000, 0.0, 44100.0/2,
      0.0277);
  
  return ret ? 0 : 1;
}


#if 0
struct timeval start_time;
void
start_timer (void)
{
  gettimeofday (&start_time, NULL);
  /*printf("start %ld.%06ld\n",start_time.tv_sec,start_time.tv_usec); */
}

void
end_timer (void)
{
  struct timeval end_time;
  double diff;

  gettimeofday (&end_time, NULL);
  /*printf("end %ld.%06ld\n",end_time.tv_sec,end_time.tv_usec); */
  diff = (end_time.tv_sec - start_time.tv_sec) +
      1e-6 * (end_time.tv_usec - start_time.tv_usec);

  printf ("time %g\n", diff);

}
#endif

static double
chirp_func (double x, double start_freq, double stop_freq)
{
  return 0.5 * sin(2*M_PI*(x)*start_freq +
      2*M_PI*(x)*(x)*(stop_freq - start_freq)*0.5);
}

static const char *
format_names[] = { "s16", "s32", "f32", "f64" };

static int
test_chirp (ResampleFormat format, int i_rate, int o_rate,
    double start_freq, double stop_freq, double expected_error)
{
  ResampleState *r;
  int i;
  double x;
  int n;
  int i_size;
  int o_size;
  void *i_buf;
  void *o_buf;
  double sum;
  int too_high;
  int too_low;

  printf("test_chirp format=%s i_rate=%d o_rate=%d start_freq=%g stop_freq=%g\n",
      format_names[format], i_rate, o_rate, start_freq, stop_freq);

  i_size = i_rate * resample_format_size(format);
  o_size = o_rate * resample_format_size(format);
  i_buf = malloc(i_size);
  o_buf = malloc(o_size);
  memset(o_buf, 0, o_size);

  switch(format){
    case RESAMPLE_FORMAT_S16:
      for (i = 0; i < i_rate; i++) {
        ((int16_t *)i_buf)[i] =
          rint (32767 * chirp_func ((double)i/i_rate, start_freq, stop_freq));
      }
      break;
    case RESAMPLE_FORMAT_S32:
      for (i = 0; i < i_rate; i++) {
        ((int32_t *)i_buf)[i] =
          rint (INT_MAX * chirp_func ((double)i/i_rate, start_freq, stop_freq));
      }
      break;
    case RESAMPLE_FORMAT_F32:
      for (i = 0; i < i_rate; i++) {
        ((float *)i_buf)[i] =
          chirp_func ((double)i/i_rate, start_freq, stop_freq);
      }
      break;
    case RESAMPLE_FORMAT_F64:
      for (i = 0; i < i_rate; i++) {
        ((double *)i_buf)[i] =
          chirp_func ((double)i/i_rate, start_freq, stop_freq);
      }
      break;
  }

  r = resample_new ();

  resample_set_input_rate (r, i_rate);
  resample_set_output_rate (r, o_rate);
  resample_set_n_channels(r, 1);
  resample_set_filter_length (r, 64);
  resample_set_format(r, format);
  resample_set_method(r, 1);

  resample_add_input_data (r, i_buf, i_size, NULL, NULL);
  resample_input_eos (r);

  n = resample_get_output_data (r, o_buf, o_size);

  sum = 0;
  switch(format){
    case RESAMPLE_FORMAT_S16:
      for (i=0;i<o_rate;i++){
        double f, g;

        f = rint (32767 *
            chirp_func ((double)i/o_rate, start_freq, stop_freq)) / 32767.0;
        g = ((int16_t *)o_buf)[i]/32767.0;
        x = g - f;
        
        sum += x * x;
      }
      break;
    case RESAMPLE_FORMAT_S32:
      for (i=0;i<o_rate;i++){
        double f, g;

        f = rint (INT_MAX *
            chirp_func ((double)i/o_rate, start_freq, stop_freq)) /
          ((double)INT_MAX);
        g = ((int32_t *)o_buf)[i]/((double)INT_MAX);
        x = g - f;
        
        sum += x * x;
      }
      break;
    case RESAMPLE_FORMAT_F32:
      for (i=0;i<o_rate;i++){
        double f, g;

        f = chirp_func ((double)i/o_rate, start_freq, stop_freq);
        g = ((float *)o_buf)[i];
        x = g - f;
        
        sum += x * x;
      }
      break;
    case RESAMPLE_FORMAT_F64:
      for (i=0;i<o_rate;i++){
        double f, g;

        f = chirp_func ((double)i/o_rate, start_freq, stop_freq);
        g = ((double *)o_buf)[i];
        x = g - f;
        
        sum += x * x;
      }
      break;
    default:
      break;
  }

  too_high = (sqrt (sum / o_rate) > 1.1 * expected_error);
  too_low = (sqrt (sum / o_rate) < 0.9 * expected_error);
  printf ("average error: %g expected %g (%s)\n\n", sqrt (sum / o_rate),
      expected_error,
      too_high ? "too high" : (too_low ? "too low" : "passed"));

  resample_free (r);
  free (i_buf);
  free (o_buf);

  return !too_high && !too_low;
}

