
/*
 * This is the standard regression test for libaudioresample.
 * It generates one second of audio data that is a frequency
 * sweep from 0 Hz to 22050 Hz at 44100 samples/s, and then
 * resamples it to 48000 samples/s.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <string.h>

#include <audioresample/resample.h>

void *get_buffer(void *priv, unsigned int size);
void start_timer(void);
void end_timer(void);

static double amplitude = 1.0;

static double i_rate = 48000.0;
static double o_rate = 44100.0;

/*#define test_func(x) 1 */
/*#define test_func(x) sin(2*M_PI*(x)*10) */
/*#define test_func(x) sin(2*M_PI*(x)*(x)*1000) */
#define test_func(x) sin(2*M_PI*(x)*(x)*12000)

float *i_buf;
float *o_buf;

//static int i_offset;
static int o_offset;

int main(int argc,char *argv[])
{
	ResampleState *r;
	int i;
	double sum10k,sum22k;
	double f;
	int n10k,n22k;
	double x;

        resample_init();

	i_buf = malloc(i_rate * sizeof(float));
	for(i=0;i<i_rate;i++){
		i_buf[i] = amplitude * test_func((double)i/i_rate);
	}

	o_buf = malloc(o_rate * sizeof(float));
	memset(o_buf,0,sizeof(float) * o_rate);

        r = resample_new ();

        resample_set_input_rate (r, i_rate);
        resample_set_output_rate (r, o_rate);
        resample_set_n_channels(r, 2);
        resample_set_filter_length (r, 64);

	start_timer();
	resample_add_input_data (r,i_buf,i_rate*2, NULL, NULL);
        resample_input_eos (r);
        o_offset = resample_get_output_data (r, o_buf, o_rate * 2);
	end_timer();

#if 1
	{
	FILE *out = fopen("out","w");
	for(i=0;i<o_rate;i++){
		f = amplitude*test_func((double)i/o_rate);
		fprintf(out,"%d %g %g %g\n",i,
			o_buf[i], f, o_buf[i]-f);
	}
	}
#endif

	sum10k=0;
	sum22k=0;
	n10k=0;
	n22k=0;
	for(i=0;i<o_rate;i++){
		f = amplitude*test_func((double)i/o_rate);
		x = o_buf[i]-f;
		if(((0.5*i)/o_rate*i_rate)<10000){
			sum10k += x*x;
			n10k++;
		}
		if(((0.5*i)/o_rate*i_rate)<22050){
			sum22k += x*x;
			n22k++;
		}
	}
	printf("average error 10k=%g 22k=%g\n",
		sqrt(sum10k/n10k),
		sqrt(sum22k/n22k));

	return 0;
}

struct timeval start_time;
void start_timer(void)
{
	gettimeofday(&start_time,NULL);
	/*printf("start %ld.%06ld\n",start_time.tv_sec,start_time.tv_usec); */
}

void end_timer(void)
{
	struct timeval end_time;
	double diff;

	gettimeofday(&end_time,NULL);
	/*printf("end %ld.%06ld\n",end_time.tv_sec,end_time.tv_usec); */
	diff = (end_time.tv_sec - start_time.tv_sec) +
		1e-6*(end_time.tv_usec - start_time.tv_usec);

	printf("time %g\n",diff);

}

